const got = require('got');
const {GoogleAuth} = require('google-auth-library');

function Ims(opts) {
  if (!(this instanceof Ims)) {
    return new Ims(opts);
  }
  this.endpoint = opts.endpoint;
}

Ims.prototype.getApplicationDefault = function getApplicationDefault() {
  return new Promise((resolve, reject) => {
    if (this.authClient) {
      return resolve(this.authClient);
    }

    const auth = new GoogleAuth({
      scopes: 'https://www.googleapis.com/auth/cloud-platform'
    });

    let authClient;

    try {
      authClient = auth.getClient()
    }
    catch(e) {
      return reject(err)
    }
    this.authClient = authClient;
    return resolve(authClient)
  });
};

Ims.prototype.getAccessToken = function getAccessToken() {
  return this.getApplicationDefault()
    .then(authClient => {
      return new Promise((resolve, reject) => {
        authClient.getAccessToken((err, accessToken) => {
          if (err) {
            return reject(err);
          }
          return resolve(accessToken);
        });
      });
    });
};

Ims.prototype.getArtifactByID = function (id) {
  return this.makeRequest('GET', `${this.endpoint}/artifacts/${id}`)
    .then(response => response.body.artifact);
};

Ims.prototype.deleteArtifactByID = function (id) {
  return this.makeRequest('DELETE', `${this.endpoint}/artifacts/${id}`);
};

Ims.prototype.getArtifacts = function (query = {}) {
  return this.makeRequest('GET', `${this.endpoint}/artifacts`, { query });
};

Ims.prototype.putArtifactByID = function (id, artifact) {
  return this.makeRequest('PUT', `${this.endpoint}/artifacts/${id}`, {
    body: { artifact },
  });
};

Ims.prototype.postEvent = function (event) {
  return this.makeRequest('POST', `${this.endpoint}/events`, {
    body: { event },
  });
};

Ims.prototype.putEventByID = function (id, event) {
  return this.makeRequest('PUT', `${this.endpoint}/events/${id}`, {
    body: { event },
  });
};

Ims.prototype.getEvent = function (id) {
  return this.makeRequest('GET', `${this.endpoint}/events/${id}`)
    .then(response => response.body.event);
};

Ims.prototype.getEvents = function (query = {}) {
  return this.makeRequest('GET', `${this.endpoint}/events`, { query } )
    .then(response => response.body);
};

Ims.prototype.deleteEvent = function (id) {
  return this.makeRequest('DELETE', `${this.endpoint}/events/${id}`);
};

// By fully qualified location. E.g repository@digest or repository:tag
Ims.prototype.getImageArtifactByFQL = function (imageLocation) {
  const [repository, digest, tag] = Ims.getDigestOrTagFromImageLocation(imageLocation);

  if (digest) {
    return this.getArtifactByID(digest);
  }
  return this.getArtifacts({
    'image.repository': repository,
    'image.tags': tag
  })
    .then(response => response.body.artifacts[0]);
}

Ims.prototype.getFunctionArtifactBySource = function (source) {
  return this.getArtifacts({
    'function.source': source
  })
    .then(response => response.body.artifacts[0]);
}

Ims.prototype.makeRequest = function (method, url, query = {}) {
  return this.getAccessToken()
    .then((accessToken) => {
      return got(url, Object.assign({}, query, {
        headers: {
          'Authorization': `Bearer ${accessToken}`,
        },
        json: true,
        method: method,
      }));
    });
}

function getDigestOrTagFromImageLocation(image) {
  let repository;
  let digest;
  let tag;

  if (image.includes('@')) {
    [repository, digest] = image.split('@');
  } else {
    [repository, tag] = image.split(':');
  }

  return [repository, digest, tag];
}

// Expose some utility functions as static methods
Ims.getDigestOrTagFromImageLocation = getDigestOrTagFromImageLocation;

module.exports = Ims;
